import json
import sys
import os
import tabulate

SUBJECTS_FOLDER = "data/"


def get_general_info(all_bad_fixes_json):
    print_dataset_summary(all_bad_fixes_json)
    table_headers = ["bad-fix", "project", "type", "crash", "classes", "methods", "#tests (bad-fix)", "#tests (fix)"]
    table_rows = []
    for bad_fix_json in all_bad_fixes_json:
        bad_fix_classes = get_classes(bad_fix_json)
        bad_fix_methods = get_methods(bad_fix_json)
        tests_bad_fix = get_number_of_tests_bad_fix(bad_fix_json)
        tests_fix = get_number_of_tests_fix(bad_fix_json)
        table_rows.append([bad_fix_json["name"], bad_fix_json["project"], bad_fix_json["type"], bad_fix_json["crash"], len(bad_fix_classes), len(bad_fix_methods), tests_bad_fix, tests_fix])
    print(tabulate.tabulate(table_rows, headers=table_headers))
    print()


def print_dataset_summary(all_bad_fixes):
    bad_fixes_amount = len(all_bad_fixes)
    print("bad fixes: " + str(bad_fixes_amount))
    coverages = len([bad_fix for bad_fix in all_bad_fixes if bad_fix["type"] == "coverage"])
    print("coverage: " + str(coverages) + " / " + "disruptions: " + str(bad_fixes_amount - coverages))
    crashes = len([bad_fix for bad_fix in all_bad_fixes if bad_fix["crash"]])
    print("crashes: " + str(crashes) + " / " + "non-crashes: " + str(bad_fixes_amount - crashes))
    with_tests_revealing_bug = len([bad_fix for bad_fix in all_bad_fixes if "test_class" in bad_fix["bad-fix"]])
    print("with bug revealing tests: " + str(with_tests_revealing_bug) + " / " + "without bug revealing tests: " + str(bad_fixes_amount - with_tests_revealing_bug))
    with_tests_revealing_bad_fix = len([bad_fix for bad_fix in all_bad_fixes if "test_class" in bad_fix["fix"]])
    print("with bad-fix revealing tests: " + str(with_tests_revealing_bad_fix) + " / " + "without bad-fix revealing tests: " + str(bad_fixes_amount - with_tests_revealing_bad_fix))
    print()


def get_bad_fix_info(bad_fix):
    bad_fix_dir = SUBJECTS_FOLDER + bad_fix

    if not os.path.exists(bad_fix_dir):
        print("Invalid bad fix, dir do not exist: " + bad_fix_dir)
        sys.exit()

    bad_fix_json_file = bad_fix_dir + '/bad-fix.json'
    f = open(bad_fix_json_file)
    bad_fix_json = json.load(f)
    print("project:", bad_fix_json["project"])
    print("repo:", bad_fix_json["repository"])
    print("issue:", bad_fix_json["issue"])
    print("type:", bad_fix_json["type"])
    print("crash:", bad_fix_json["crash"])
    print()
    print("-- Bug --")
    print("  revision:", bad_fix_json["bug"]["commit"])
    print("")
    print("-- Bad fix --")
    print("  revision:", bad_fix_json["bad-fix"]["commit"])
    print("  commit link:", bad_fix_json["repository"] + "/commit/" + bad_fix_json["bad-fix"]["commit"])
    if "changes" in bad_fix_json["bad-fix"]:
        print("  changes:")
        for change in bad_fix_json["bad-fix"]["changes"]:
            print("    class:", change["class"])
            print("    methods:", change["methods"])
    if "test_class" in bad_fix_json["bad-fix"]:
        print("  test class:", bad_fix_json["bad-fix"]["test_class"])
        print("  tests:", bad_fix_json["bad-fix"]["tests"])
    if bad_fix_json["fix"]:
        print()
        print("-- Good fix --")
        print("  revision:", bad_fix_json["fix"]["commit"])
        print("  commit link:", bad_fix_json["repository"] + "/commit/" + bad_fix_json["fix"]["commit"])
        if "changes" in bad_fix_json["fix"]:
            print("  changes:")
            for change in bad_fix_json["fix"]["changes"]:
                print("    class:", change["class"])
                print("    methods:", change["methods"])
        if "test_class" in bad_fix_json["fix"]:
            print("  test class:", bad_fix_json["fix"]["test_class"])
            print("  tests:", bad_fix_json["fix"]["tests"])
    print()


def get_classes(bad_fix_json):
    classes = set()
    for change in bad_fix_json["bad-fix"]["changes"]:
        classes.add(change["class"])
    for change in bad_fix_json["fix"]["changes"]:
        classes.add(change["class"])
    return classes


def get_methods(bad_fix_json):
    methods = set()
    for change in bad_fix_json["bad-fix"]["changes"]:
        for method in change["methods"]:
            methods.add(method)
    for change in bad_fix_json["fix"]["changes"]:
        for method in change["methods"]:
            methods.add(method)
    return methods


def get_number_of_tests_bad_fix(bad_fix_json):
    if "test_class" in bad_fix_json["bad-fix"]:
        return len(bad_fix_json["bad-fix"]["tests"])
    return 0


def get_number_of_tests_fix(bad_fix_json):
    if "test_class" in bad_fix_json["fix"]:
        return len(bad_fix_json["fix"]["tests"])
    return 0


def get_info(bad_fix):
    if bad_fix is None:
        all_bad_fixes = sorted(os.listdir(SUBJECTS_FOLDER))
        all_jsons_files = [SUBJECTS_FOLDER + bad_fix + '/bad-fix.json' for bad_fix in all_bad_fixes]
        all_bad_fixes_json = [json.load(open(bad_fix_json_file)) for bad_fix_json_file in all_jsons_files]
        print("> General dataset info")
        get_general_info(all_bad_fixes_json)
    else:
        print("> General info for bad fix:", bad_fix)
        get_bad_fix_info(bad_fix)


def get_formatted_info(bad_fix, output_format):
    if bad_fix is None:
        print("Printing formatted info for all bad fixes: " + output_format)
        print()
        all_bad_fixes = sorted(os.listdir(SUBJECTS_FOLDER))
        all_jsons_files = [SUBJECTS_FOLDER + bad_fix + '/bad-fix.json' for bad_fix in all_bad_fixes]
        all_bad_fixes_json = [json.load(open(bad_fix_json_file)) for bad_fix_json_file in all_jsons_files]
        if output_format == "csv":
            print("bad-fix,project,type,crash,classes,methods,tests_bad_fix,tests_fix")
            for bad_fix_json in all_bad_fixes_json:
                print(bad_fix_json["name"] + "," + bad_fix_json["project"] + "," + bad_fix_json["type"] + "," + str(bad_fix_json["crash"]) + "," + str(len(get_classes(bad_fix_json))) + "," + str(len(get_methods(bad_fix_json))) + "," + str(get_number_of_tests_bad_fix(bad_fix_json)) + "," + str(get_number_of_tests_fix(bad_fix_json)))
        print()
    else:
        print("Printing formatted info for a specific bad fix still not supported")
        sys.exit()
