import sys
import os
import json
import csv

output_dir = "inspector-output"


def reopened_issues(url, key):
    """
    Get the list of reopened issues from a jira project
    """
    print("Getting the list of reopened issues from the jira project:", key)
    print()
    reopened_issues_url = f'https://{url}/jira/rest/api/latest/search\?jql\=project\={key}%20and%20status%20was%20Reopened\&maxResults\=1000'
    stream = os.popen("curl " + reopened_issues_url)
    all_issues = json.loads(stream.read())
    print("reopened issues:", all_issues["total"])
    print("issues array:", str(len(all_issues["issues"])))
    print()
    return all_issues


def inspect_all_issues(project_repo, all_issues):
    """
    Inspect all issues from a jira project, by searching in the corresponding github repository
    """
    print("Inspecting all issues from the jira project")
    issues_to_check = []
    for issue in all_issues["issues"]:
        issue_key = issue["key"]
        github_query = "gh api -H \"Accept: application/vnd.github+json\" --method GET /search/commits\?q=" + issue_key + "+repo:" + project_repo
        stream = os.popen(github_query)
        commits_mentioning_issue = json.loads(stream.read())
        if "total_count" not in commits_mentioning_issue:
            print("finish due to API rate limit exceeded")
            break
        if commits_mentioning_issue["total_count"] > 1:
            print("issue with at least two commits: " + issue_key)
            link_to_commits = "https://github.com/" + project_repo + "/search?q=" + issue_key + "&type=commits"
            issues_to_check.append([project_repo, issue_key, link_to_commits])
    print("total issues to check: ", len(issues_to_check))
    print()
    return issues_to_check


def save_to_file(key, issues_to_check):
    """
    Saving the issues to check to a file
    """
    output_file = output_dir + "/" + key + "-issues.csv"
    print("output file: "+output_file)
    os.makedirs(os.path.dirname(output_file), exist_ok=True)
    csv_f = open(output_file, "a", encoding='UTF8')
    writer = csv.writer(csv_f)
    writer.writerow(["project", "issue", "link"])
    for issue_row in issues_to_check:
        writer.writerow(issue_row)
    print()


def inspect_jira_repository(name, key, url):
    """
    Inspect a jira repository for issues/commits
    """
    print("key:", key)
    print("url:", url)
    print()
    all_issues = reopened_issues(url, key)
    issues_to_check = inspect_all_issues(name, all_issues)
    save_to_file(key, issues_to_check)
    sys.exit()
