import sys
import json
import os
import subprocess
import shutil

SUBJECTS_FOLDER = "data/"
TMP_FOLDER = "tmp/"


def setup_bad_fix(bad_fix, version):
    print("> Setting up bad fix")
    bad_fix_dir = SUBJECTS_FOLDER + bad_fix
    print("dir: " + bad_fix_dir)
    print("version: " + version)

    if not os.path.exists(bad_fix_dir):
        print("Invalid bad fix")
        sys.exit()

    bad_fix_json_file = bad_fix_dir + '/bad-fix.json'
    f = open(bad_fix_json_file)
    bad_fix_json = json.load(f)

    project = bad_fix_json["project"]
    repository = bad_fix_json["repository"]
    print("project: " + project)
    print("repository: " + repository)
    print("issue: " + bad_fix_json["issue"])
    print()

    buggy_hash = bad_fix_json["bug"]["commit"]
    bad_fix_hash = bad_fix_json["bad-fix"]["commit"]
    fix_hash = bad_fix_json["fix"]["commit"]
    root_dir = os.getcwd()

    if version == "fix":
        print("Compiling fixed version: " + fix_hash)
        target_dir_fixed = TMP_FOLDER + bad_fix + "/fixed"
        print("target dir: " + target_dir_fixed)
        if not os.path.exists(target_dir_fixed):
            os.makedirs(target_dir_fixed)
            os.chdir(target_dir_fixed)
            subprocess.call('git clone ' + repository, shell=True)
            os.chdir(project)
            subprocess.call('git checkout -f ' + fix_hash, shell=True)

        else:
            os.chdir(target_dir_fixed + "/" + project)

        build_command = prepare_for_build(root_dir, bad_fix_json, bad_fix_dir, "bad-fix")
        print("build command: " + build_command)
        subprocess.call(build_command, shell=True)
        os.chdir(root_dir)
        print()
        exit()


    print("Compiling buggy version: " + buggy_hash)
    target_dir_buggy = TMP_FOLDER + bad_fix + "/buggy"
    print("target dir: " + target_dir_buggy)

    if not os.path.exists(target_dir_buggy):
        os.makedirs(target_dir_buggy)
        os.chdir(target_dir_buggy)
        subprocess.call('git clone ' + repository, shell=True)
        os.chdir(project)
        subprocess.call('git checkout ' + buggy_hash, shell=True)

    else:
        os.chdir(target_dir_buggy + "/" + project)

    build_command = prepare_for_build(root_dir, bad_fix_json, bad_fix_dir, "bug")
    print("build command: " + build_command)
    subprocess.call(build_command, shell=True)
    os.chdir(root_dir)
    print()

    print("Compiling bad fix version: " + bad_fix_hash)
    target_dir_badfix = TMP_FOLDER + bad_fix + "/badfix"
    print("target dir: " + target_dir_badfix)
    if not os.path.exists(target_dir_badfix):
        shutil.copytree(target_dir_buggy, target_dir_badfix)
        os.chdir(target_dir_badfix + "/" + project)
        subprocess.call('git checkout -f ' + bad_fix_hash, shell=True)
    else:
        os.chdir(target_dir_badfix + "/" + project)

    build_command = prepare_for_build(root_dir, bad_fix_json, bad_fix_dir, "bad-fix")
    print("build command: " + build_command)
    subprocess.call(build_command, shell=True)
    os.chdir(root_dir)
    print()


def prepare_for_build(root_dir, bad_fix_json, bad_fix_dir, version):
    build_command = "mvn clean compile"
    if "build" in bad_fix_json:
        tool = bad_fix_json["build"]["tool"]
        if "command" in bad_fix_json["build"]:
            build_command = bad_fix_json["build"]["command"]
            if "with_deps" in bad_fix_json["build"]:
                build_command = bad_fix_json["build"]["with_deps"]
        if "include_lib" in bad_fix_json["build"] and bad_fix_json["build"]["include_lib"]:
            if not os.path.exists("lib"):
                os.makedirs("lib")
            subprocess.call('cp ' + root_dir + '/' + bad_fix_dir + '/lib/* lib/', shell=True)
        if "use_own_build_file" in bad_fix_json["build"]:
            if tool == "maven":
                subprocess.call('cp ' + root_dir + '/' + bad_fix_dir + '/pom.xml.ok pom.xml', shell=True)
            elif tool == "ant":
                subprocess.call('cp ' + root_dir + '/' + bad_fix_dir + '/build.xml.ok build.xml', shell=True)
            elif tool == "gradle":
                subprocess.call('cp ' + root_dir + '/' + bad_fix_dir + '/build.gradle.ok build.gradle', shell=True)
            else:
                print("Unsupported build tool")
                sys.exit()
        if "copy_files_for_build" in bad_fix_json["build"]:
            for file in bad_fix_json["build"]["copy_files_for_build"]:
                if "version" in file and file["version"] != version:
                    continue
                subprocess.call('cp ' + root_dir + '/' + bad_fix_dir + '/' + file["from"] + ' ' + file["to"], shell=True)
        if "base_dir" in bad_fix_json["build"]:
            print("changing dir to: " + bad_fix_json["build"]["base_dir"])
            os.chdir(bad_fix_json["build"]["base_dir"])

    return build_command
